#!/usr/bin/env python

import os
import os.path

import createrepo_c as cr

REPO_PATH = "repo/"


def print_package_info(pkg):
    def print_pcors(lst, requires=False):
        for item in lst:
            print "    Name: %s" % item[cr.PCOR_ENTRY_NAME]
            print "    Flags: %s" % item[cr.PCOR_ENTRY_FLAGS]
            print "    Epoch: %s" % item[cr.PCOR_ENTRY_EPOCH]
            print "    Version: %s" % item[cr.PCOR_ENTRY_VERSION]
            print "    Release: %s" % item[cr.PCOR_ENTRY_RELEASE]
            if requires:
                print "    Pre: %s" % item[cr.PCOR_ENTRY_PRE]
            print "    +-----------------------------------+"

    def print_files(lst):
        for item in lst:
            print "    Name: %s" % item[cr.FILE_ENTRY_NAME]
            print "    Path: %s" % item[cr.FILE_ENTRY_PATH]
            print "    Type: %s" % item[cr.FILE_ENTRY_TYPE]
            print "    +-----------------------------------+"

    def print_changelogs(lst):
        for item in lst:
            print "    Author: %s" % item[cr.CHANGELOG_ENTRY_AUTHOR]
            print "    Date: %s" % item[cr.CHANGELOG_ENTRY_DATE]
            print "    Changelog: %s" % item[cr.CHANGELOG_ENTRY_CHANGELOG]
            print "    +-----------------------------------+"

    print "+=======================================+"
    print "  %s" % pkg.name
    print "+=======================================+"
    print "NEVRA: %s" % pkg.nevra()
    print "NVRA: %s" % pkg.nvra()
    print "Name: %s" % pkg.name
    print "Checksum (pkgId): %s" % pkg.pkgId
    print "Checksum type: %s" % pkg.checksum_type
    print "Arch: %s" % pkg.arch
    print "Version: %s" % pkg.version
    print "Epoch: %s" % pkg.epoch
    print "Release: %s" % pkg.release
    print "Summary: %s" % pkg.summary
    print "Description: %s" % pkg.description
    print "URL: %s" % pkg.url
    print "Time file: %s" % pkg.time_file
    print "Time build: %s" % pkg.time_build
    print "License: %s" % pkg.rpm_license
    print "Vendor: %s" % pkg.rpm_vendor
    print "Group: %s" % pkg.rpm_group
    print "Buildhost: %s" % pkg.rpm_buildhost
    print "Source RPM: %s" % pkg.rpm_sourcerpm
    print "Header start: %s" % pkg.rpm_header_start
    print "Header end: %s" % pkg.rpm_header_end
    print "Packager: %s" % pkg.rpm_packager
    print "Size package: %s" % pkg.size_package
    print "Size installed: %s" % pkg.size_installed
    print "Size archive: %s" % pkg.size_archive
    print "Location href: %s" % pkg.location_href
    print "Location base: %s" % pkg.location_base
    print "Requires:"
    print_pcors(pkg.requires, requires=True)
    print "Provides:"
    print_pcors(pkg.provides)
    print "Conflicts:"
    print_pcors(pkg.conflicts)
    print "Obsoletes:"
    print_pcors(pkg.obsoletes)
    print "Files:"
    print_files(pkg.files)
    print "Changelogs:"
    print_changelogs(pkg.changelogs)

def first_method():
    """Use of this method is discouraged."""
    md = cr.Metadata()
    md.locate_and_load_xml(REPO_PATH)
    for key in md.keys():
        pkg = md.get(key)
        print_package_info(pkg)

def second_method():
    """Prefered method for repodata parsing.

    Important callbacks for repodata parsing:

    newpkgcb
    --------
    Via newpkgcb (Package callback) you could directly
    affect if the current package element shoud be parsed
    or not. This decision could be based on
    three values that are available as attributtes
    in the <package> element. This values are:
     - pkgId (package checksum)
     - name (package name)
     - arch (package architecture)
    (Note: This is applicable only for filelists.xml and other.xml,
     primary.xml doesn't contain this information in <package> element)

    If newpkgcb returns a package object, the parsed data
    will be loaded to this package object. If it returns a None,
    package element is skiped.

    This could help you to reduce a memory requirements because
    non wanted packages could be skiped without need to
    store them into the memory.

    If no newpkgcb is specified, default callback returning
    a new package object is used.

    pkgcb
    -----
    Callback called when a <package> element parsing is done.
    Its argument is a package object that has been previously
    returned by the newpkgcb.
    This function should return True if parsing should continue
    or False if parsing should be interrupted.

    Note: Both callbacks are optional, BUT at least one
          MUST be used (newpkgcb or pkgcb)!

    warningcb
    ---------
    Warning callbacks is called when a non-fatal oddity of prased XML
    is detected.
    If True is returned, parsing continues. If return value is False,
    parsing is terminated.
    This callback is optional.
    """

    primary_xml_path   = None
    filelists_xml_path = None
    other_xml_path     = None

    #
    # repomd.xml parsing
    #

    # Parse repomd.xml to get paths (1. Method - Repomd object based)
    #   Pros: Easy to use
    repomd = cr.Repomd(os.path.join(REPO_PATH, "repodata/repomd.xml"))

    # Parse repomd.xml (2. Method - Parser based)
    #   Pros: Warning callback could be specified
    def warningcb(warning_type, message):
        """Optional callback for warnings about
        wierd stuff and formatting in XML.

        :param warning_type: Integer value. One from
                             the XML_WARNING_* constants.
        :param message: String message.
        """
        print "PARSER WARNING: %s" % message
        return True

    repomd2 = cr.Repomd()
    cr.xml_parse_repomd(os.path.join(REPO_PATH, "repodata/repomd.xml"),
                                     repomd2, warningcb)

    # Get stuff we need
    #   (repomd or repomd2 could be used, both have the same values)
    for record in repomd.records:
        if record.type == "primary":
            primary_xml_path = record.location_href
        elif record.type == "filelists":
            filelists_xml_path = record.location_href
        elif record.type == "other":
            other_xml_path = record.location_href


    #
    # Main XML metadata parsing (primary, filelists, other)
    #

    packages = {}

    def pkgcb(pkg):
        # Called when whole package entry in xml is parsed
        packages[pkg.pkgId] = pkg

    def newpkgcb(pkgId, name, arch):
        # Called when new package entry is encountered
        # And only opening <package> element is parsed
        # This function has to return a package to which
        # parsed data will be added or None if this package
        # should be skiped.
        return packages.get(pkgId, None)

    # Option do_files tells primary parser to skip <file> element of package.
    # If you plan to parse filelists.xml after the primary.xml, always
    # set do_files to False.
    cr.xml_parse_primary(os.path.join(REPO_PATH, primary_xml_path),
                         pkgcb=pkgcb,
                         do_files=False,
                         warningcb=warningcb)

    cr.xml_parse_filelists(os.path.join(REPO_PATH, filelists_xml_path),
                           newpkgcb=newpkgcb,
                           warningcb=warningcb)

    cr.xml_parse_other(os.path.join(REPO_PATH, other_xml_path),
                       newpkgcb=newpkgcb,
                       warningcb=warningcb)

    for pkg in packages.itervalues():
        print_package_info(pkg)


if __name__ == "__main__":
    print '"All in one shot" method:'
    first_method()

    print

    print "Callback based method:"
    second_method()




import re
import unittest
import shutil
import tempfile
import os.path
import createrepo_c as cr

from .fixtures import *

class TestCaseRepomd(unittest.TestCase):

    def setUp(self):
        self.tmpdir = tempfile.mkdtemp(prefix="createrepo_ctest-")
        self.FN_00 = "primary.xml.gz"
        self.FN_01 = "primary.xml"
        self.path00 = os.path.join(self.tmpdir, self.FN_00)
        self.path01 = os.path.join(self.tmpdir, self.FN_01)

    def tearDown(self):
        shutil.rmtree(self.tmpdir)

    def xxx_repomdrecord_fill(self):
        shutil.copyfile(REPO_00_PRIXML, self.path00)
        self.assertTrue(os.path.exists(self.path00))

        rec = cr.RepomdRecord("primary", self.path00)
        self.assertTrue(rec)
        rec.fill(cr.SHA256)
        rec.rename_file()

        # Filename shoud contain a (valid) checksum
        self.assertEqual(os.listdir(self.tmpdir),
            ['1cb61ea996355add02b1426ed4c1780ea75ce0c04c5d1107c025c3fbd7d8bcae-primary.xml.gz'])

    def test_repomd(self):
        shutil.copyfile(REPO_00_PRIXML, self.path00)
        self.assertTrue(os.path.exists(self.path00))

        md = cr.Repomd()
        self.assertTrue(md)

        xml = md.xml_dump()
        # Revision shoud be current Unix time
        self.assertTrue(re.search(r"<revision>[0-9]+</revision>", xml))

        self.assertEqual(md.revision, None)
        md.set_revision("foobar")
        self.assertEqual(md.revision, "foobar")

        self.assertEqual(md.repoid, None);
        md.set_repoid("barid", "sha256")
        self.assertEqual(md.repoid, "barid")

        self.assertEqual(md.contenthash, None);
        md.set_contenthash("fooid", "sha256")
        self.assertEqual(md.contenthash, "fooid")

        self.assertEqual(md.distro_tags, [])
        md.add_distro_tag("tag1")
        md.add_distro_tag("tag2", "cpeid1")
        md.add_distro_tag("tag3", cpeid="cpeid2")
        self.assertEqual(md.distro_tags,
                [(None, 'tag1'),
                 ('cpeid1', 'tag2'),
                 ('cpeid2', 'tag3')])

        self.assertEqual(md.repo_tags, [])
        md.add_repo_tag("repotag")
        self.assertEqual(md.repo_tags, ['repotag'])

        self.assertEqual(md.content_tags, [])
        md.add_content_tag("contenttag")
        self.assertEqual(md.content_tags, ['contenttag'])

        self.assertEqual(md.records, [])

        xml = md.xml_dump()
        self.assertEqual(xml,
"""<?xml version="1.0" encoding="UTF-8"?>
<repomd xmlns="http://linux.duke.edu/metadata/repo" xmlns:rpm="http://linux.duke.edu/metadata/rpm">
  <revision>foobar</revision>
  <repoid type="sha256">barid</repoid>
  <contenthash type="sha256">fooid</contenthash>
  <tags>
    <content>contenttag</content>
    <repo>repotag</repo>
    <distro>tag1</distro>
    <distro cpeid="cpeid1">tag2</distro>
    <distro cpeid="cpeid2">tag3</distro>
  </tags>
</repomd>
""")

        rec = cr.RepomdRecord("primary", self.path00)
        rec.fill(cr.SHA256)
        rec.timestamp = 1
        rec.location_base = "http://foo/"
        md.set_record(rec)

        self.assertEqual(len(md.records), 1)

        md.set_record(rec)

        self.assertEqual(len(md.records), 1)

        md.repoid = None
        md.contenthash = None

        xml = md.xml_dump()
        self.assertEqual(xml,
"""<?xml version="1.0" encoding="UTF-8"?>
<repomd xmlns="http://linux.duke.edu/metadata/repo" xmlns:rpm="http://linux.duke.edu/metadata/rpm">
  <revision>foobar</revision>
  <tags>
    <content>contenttag</content>
    <repo>repotag</repo>
    <distro>tag1</distro>
    <distro cpeid="cpeid1">tag2</distro>
    <distro cpeid="cpeid2">tag3</distro>
  </tags>
  <data type="primary">
    <checksum type="sha256">1cb61ea996355add02b1426ed4c1780ea75ce0c04c5d1107c025c3fbd7d8bcae</checksum>
    <open-checksum type="sha256">e1e2ffd2fb1ee76f87b70750d00ca5677a252b397ab6c2389137a0c33e7b359f</open-checksum>
    <location href="repodata/primary.xml.gz" xml:base="http://foo/"/>
    <timestamp>1</timestamp>
    <size>134</size>
    <open-size>167</open-size>
  </data>
</repomd>
""")

    def test_repomd_with_path_in_constructor_repo01(self):

        repomd = cr.Repomd(REPO_01_REPOMD)
        self.assertEqual(repomd.revision, "1334667230")
        self.assertEqual(repomd.repo_tags, [])
        self.assertEqual(repomd.distro_tags, [])
        self.assertEqual(repomd.content_tags, [])
        self.assertEqual(len(repomd.records), 3)

    def test_repomd_indexing_and_iteration_repo01(self):
        repomd = cr.Repomd(REPO_01_REPOMD)

        types = []
        for rec in repomd:
            types.append(rec.type)
        self.assertEqual(types, ['filelists', 'other', 'primary'])

        rec = repomd["primary"]
        self.assertEqual(rec.type, "primary")

        self.assertRaises(KeyError, repomd.__getitem__, "foobar")

        self.assertTrue("primary" in repomd)



import unittest
import shutil
import tempfile
import os.path
import createrepo_c as cr

from .fixtures import *

class TestCaseRepomdRecord(unittest.TestCase):

    # TODO:
    #  - Test rename_file() for files with checksum

    def setUp(self):
        self.tmpdir = tempfile.mkdtemp(prefix="createrepo_ctest-")
        self.FN_00 = "primary.xml.gz"
        self.FN_01 = "primary.xml"
        self.FN_02 = "primary.xml.zck"
        self.path00 = os.path.join(self.tmpdir, self.FN_00)
        self.path01 = os.path.join(self.tmpdir, self.FN_01)
        self.path02 = os.path.join(self.tmpdir, self.FN_02)

    def tearDown(self):
        shutil.rmtree(self.tmpdir)

    def test_repomdrecord_fill(self):
        shutil.copyfile(REPO_00_PRIXML, self.path00)
        self.assertTrue(os.path.exists(self.path00))

        rec = cr.RepomdRecord("primary", self.path00)
        self.assertTrue(rec)

        self.assertEqual(rec.location_real, self.path00)
        self.assertEqual(rec.location_href, "repodata/primary.xml.gz")
        self.assertEqual(rec.location_base, None)
        self.assertEqual(rec.checksum, None)
        self.assertEqual(rec.checksum_type, None)
        self.assertEqual(rec.checksum_open, None)
        self.assertEqual(rec.checksum_open_type, None)
        self.assertEqual(rec.timestamp, 0)

        self.assertEqual(rec.size, 0)
        self.assertEqual(rec.size_open, -1)
        self.assertEqual(rec.db_ver, 0)

        rec.fill(cr.SHA256)

        self.assertEqual(rec.location_real, self.path00)
        self.assertEqual(rec.location_href, "repodata/primary.xml.gz")
        self.assertEqual(rec.location_base, None)
        self.assertEqual(rec.checksum, "1cb61ea996355add02b1426ed4c1780ea75ce0c04c5d1107c025c3fbd7d8bcae")
        self.assertEqual(rec.checksum_type, "sha256")
        self.assertEqual(rec.checksum_open, "e1e2ffd2fb1ee76f87b70750d00ca5677a252b397ab6c2389137a0c33e7b359f")
        self.assertEqual(rec.checksum_open_type, "sha256")
        self.assertTrue(rec.timestamp > 0)
        self.assertEqual(rec.size, 134)
        self.assertEqual(rec.size_open, 167)

        rec.rename_file()

        shutil.copyfile(REPO_00_PRIZCK, self.path02)
        self.assertTrue(os.path.exists(self.path02))

        zrc = cr.RepomdRecord("primary_zck", self.path02)
        self.assertTrue(zrc)

        self.assertEqual(zrc.location_real, self.path02)
        self.assertEqual(zrc.location_href, "repodata/primary.xml.zck")
        self.assertEqual(zrc.location_base, None)
        self.assertEqual(zrc.checksum, None)
        self.assertEqual(zrc.checksum_type, None)
        self.assertEqual(zrc.checksum_open, None)
        self.assertEqual(zrc.checksum_open_type, None)
        self.assertEqual(zrc.checksum_header, None)
        self.assertEqual(zrc.checksum_header_type, None)
        self.assertEqual(zrc.timestamp, 0)

        self.assertEqual(zrc.size, 0)
        self.assertEqual(zrc.size_open, -1)
        self.assertEqual(zrc.size_header, -1)

        if cr.HAS_ZCK == 0:
            filelist = os.listdir(self.tmpdir)
            filelist.sort()
            self.assertEqual(filelist,
                ['1cb61ea996355add02b1426ed4c1780ea75ce0c04c5d1107c025c3fbd7d8bcae-primary.xml.gz',
                 'primary.xml.zck'])
            return

        zrc.fill(cr.SHA256)

        self.assertEqual(zrc.location_real, self.path02)
        self.assertEqual(zrc.location_href, "repodata/primary.xml.zck")
        self.assertEqual(zrc.location_base, None)
        self.assertEqual(zrc.checksum, "e0ac03cd77e95e724dbf90ded0dba664e233315a8940051dd8882c56b9878595")
        self.assertEqual(zrc.checksum_type, "sha256")
        self.assertEqual(zrc.checksum_open, "e1e2ffd2fb1ee76f87b70750d00ca5677a252b397ab6c2389137a0c33e7b359f")
        self.assertEqual(zrc.checksum_open_type, "sha256")
        self.assertEqual(zrc.checksum_header, "243baf7c02f5241d46f2e8c237ebc7ea7e257ca993d9cfe1304254c7ba7f6546")
        self.assertEqual(zrc.checksum_header_type, "sha256")
        self.assertTrue(zrc.timestamp > 0)
        self.assertEqual(zrc.size, 269)
        self.assertEqual(zrc.size_open, 167)
        self.assertEqual(zrc.size_header, 132)
        self.assertEqual(zrc.db_ver, 10)

        zrc.rename_file()

        # Filename shoud contain a (valid) checksum
        filelist = os.listdir(self.tmpdir)
        filelist.sort()
        self.assertEqual(filelist,
            ['1cb61ea996355add02b1426ed4c1780ea75ce0c04c5d1107c025c3fbd7d8bcae-primary.xml.gz',
             'e0ac03cd77e95e724dbf90ded0dba664e233315a8940051dd8882c56b9878595-primary.xml.zck'])

    def test_repomdrecord_setters(self):
        shutil.copyfile(REPO_00_PRIXML, self.path00)
        self.assertTrue(os.path.exists(self.path00))

        rec = cr.RepomdRecord("primary", self.path00)
        self.assertTrue(rec)

        rec.fill(cr.SHA256)

        self.assertEqual(rec.type, "primary")
        self.assertEqual(rec.location_real, self.path00)
        self.assertEqual(rec.location_href, "repodata/primary.xml.gz")
        self.assertEqual(rec.checksum, "1cb61ea996355add02b1426ed4c1780ea75ce0c04c5d1107c025c3fbd7d8bcae")
        self.assertEqual(rec.checksum_type, "sha256")
        self.assertEqual(rec.checksum_open, "e1e2ffd2fb1ee76f87b70750d00ca5677a252b397ab6c2389137a0c33e7b359f")
        self.assertEqual(rec.checksum_open_type, "sha256")
        self.assertTrue(rec.timestamp > 0)
        self.assertEqual(rec.size, 134)
        self.assertEqual(rec.size_open, 167)
        self.assertEqual(rec.db_ver, 10)

        # Set new values

        rec.type = "foo"
        rec.location_href = "repodata/foo.xml.gz"
        rec.checksum = "foobar11"
        rec.checksum_type = "foo1"
        rec.checksum_open = "foobar22"
        rec.checksum_open_type = "foo2"
        rec.timestamp = 123
        rec.size = 456
        rec.size_open = 789
        rec.db_ver = 11

        # Check

        self.assertEqual(rec.type, "foo")
        self.assertEqual(rec.location_real, self.path00)
        self.assertEqual(rec.location_href, "repodata/foo.xml.gz")
        self.assertEqual(rec.checksum, "foobar11")
        self.assertEqual(rec.checksum_type, "foo1")
        self.assertEqual(rec.checksum_open, "foobar22")
        self.assertEqual(rec.checksum_open_type, "foo2")
        self.assertEqual(rec.timestamp, 123)
        self.assertEqual(rec.size, 456)
        self.assertEqual(rec.size_open, 789)
        self.assertEqual(rec.db_ver, 11)

    def test_repomdrecord_compress_and_fill(self):
        open(self.path01, "w").write("foobar\ncontent\nhh\n")
        self.assertTrue(os.path.exists(self.path01))

        rec = cr.RepomdRecord("primary", self.path01)
        self.assertTrue(rec)
        rec_compressed = rec.compress_and_fill(cr.SHA256, cr.GZ_COMPRESSION)

        # A new compressed file shoud be created
        self.assertEqual(sorted(os.listdir(self.tmpdir)),
            sorted(['primary.xml.gz', 'primary.xml']))

        rec.rename_file()
        rec_compressed.rename_file()

        # Filename shoud contain a (valid) checksum
        self.assertEqual(sorted(os.listdir(self.tmpdir)),
            sorted(['10091f8e2e235ae875cb18c91c443891c7f1a599d41f44d518e8af759a6c8109-primary.xml.gz',
                    'b33fc63178d852333a826385bc15d9b72cb6658be7fb927ec28c4e40b5d426fb-primary.xml']))

    def test_repomdrecord_load_contentstat(self):
        rec = cr.RepomdRecord("primary", None)
        self.assertTrue(rec)

        stat = cr.ContentStat(cr.SHA256)
        stat.checksum = "foobar"
        stat.checksum_type = cr.SHA256
        stat.size = 123

        self.assertEqual(rec.checksum_open, None)
        self.assertEqual(rec.checksum_open_type, None)
        self.assertEqual(rec.size, 0)

        rec.load_contentstat(stat);

        self.assertEqual(rec.checksum_open, "foobar")
        self.assertEqual(rec.checksum_open_type, "sha256")
        self.assertEqual(rec.size_open, 123)

